@@ -1,9 +1,32 @@
 #include <stdlib.h>
 #include <string.h>
+#ifndef WIN32
+#include "config.h"
+#endif
+
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#endif
+
+#define         _ISOC9X_SOURCE  1
+#define         _ISOC99_SOURCE  1
+#define         __USE_ISOC99    1
+#define         __USE_ISOC9X    1
+
 #include <math.h>
 
 #include "ladspa.h"
 
+#ifdef WIN32
+#define _WINDOWS_DLL_EXPORT_ __declspec(dllexport)
+int bIsFirstTime = 1; 
+void _init(); // forward declaration
+#else
+#define _WINDOWS_DLL_EXPORT_ 
+#endif
+
+#line 8 "retro_flange_1208.xml"
+
 #include "ladspa-util.h"
 
 #define BASE_BUFFER 0.001 // Base buffer length (s)
@@ -25,21 +48,21 @@ inline void rc_window(float *window, float *meta_window, long law_p, long max_la
 
 inline LADSPA_Data sat(LADSPA_Data x, float q,  float dist) {
         if (x == q) {
-                return 1.0f / dist + q / (1.0f - exp(dist * q));
+                return 1.0f / dist + q / (1.0f - f_exp(dist * q));
         }
-        return ((x - q) / (1.0f - exp(-dist * (x - q))) + q /
-         (1.0f - exp(dist * q)));
+        return ((x - q) / (1.0f - f_exp(-dist * (x - q))) + q /
+         (1.0f - f_exp(dist * q)));
 }
 
-#define RETROFLANGE_DELAY_DEPTH        0
+#define RETROFLANGE_DELAY_DEPTH_AVG    0
 #define RETROFLANGE_LAW_FREQ           1
 #define RETROFLANGE_INPUT              2
 #define RETROFLANGE_OUTPUT             3
 
-LADSPA_Descriptor *retroFlangeDescriptor = NULL;
+static LADSPA_Descriptor *retroFlangeDescriptor = NULL;
 
 typedef struct {
-	LADSPA_Data *delay_depth;
+	LADSPA_Data *delay_depth_avg;
 	LADSPA_Data *law_freq;
 	LADSPA_Data *input;
 	LADSPA_Data *output;
@@ -65,7 +88,15 @@ typedef struct {
 	LADSPA_Data run_adding_gain;
 } RetroFlange;
 
+_WINDOWS_DLL_EXPORT_
 const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
+
+#ifdef WIN32
+	if (bIsFirstTime) {
+		_init();
+		bIsFirstTime = 0;
+	}
+#endif
 	switch (index) {
 	case 0:
 		return retroFlangeDescriptor;
@@ -74,11 +105,67 @@ const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
 	}
 }
 
-void cleanupRetroFlange(LADSPA_Handle instance) {
+static void activateRetroFlange(LADSPA_Handle instance) {
+	RetroFlange *plugin_data = (RetroFlange *)instance;
+	LADSPA_Data *buffer = plugin_data->buffer;
+	long buffer_size = plugin_data->buffer_size;
+	long count = plugin_data->count;
+	LADSPA_Data *delay_line = plugin_data->delay_line;
+	int delay_line_length = plugin_data->delay_line_length;
+	int delay_pos = plugin_data->delay_pos;
+	LADSPA_Data last_in = plugin_data->last_in;
+	int last_law_p = plugin_data->last_law_p;
+	int last_phase = plugin_data->last_phase;
+	int law_pos = plugin_data->law_pos;
+	float *law_tbl = plugin_data->law_tbl;
+	int max_law_p = plugin_data->max_law_p;
+	float *meta_window = plugin_data->meta_window;
+	float phase = plugin_data->phase;
+	long sample_rate = plugin_data->sample_rate;
+	float *window = plugin_data->window;
+	LADSPA_Data z0 = plugin_data->z0;
+	LADSPA_Data z1 = plugin_data->z1;
+	LADSPA_Data z2 = plugin_data->z2;
+#line 76 "retro_flange_1208.xml"
+	memset(delay_line, 0, sizeof(float) * delay_line_length);
+	memset(buffer, 0, sizeof(LADSPA_Data) * buffer_size);
+	z0 = 0.0f;
+	z1 = 0.0f;
+	z2 = 0.0f;
+	plugin_data->buffer = buffer;
+	plugin_data->buffer_size = buffer_size;
+	plugin_data->count = count;
+	plugin_data->delay_line = delay_line;
+	plugin_data->delay_line_length = delay_line_length;
+	plugin_data->delay_pos = delay_pos;
+	plugin_data->last_in = last_in;
+	plugin_data->last_law_p = last_law_p;
+	plugin_data->last_phase = last_phase;
+	plugin_data->law_pos = law_pos;
+	plugin_data->law_tbl = law_tbl;
+	plugin_data->max_law_p = max_law_p;
+	plugin_data->meta_window = meta_window;
+	plugin_data->phase = phase;
+	plugin_data->sample_rate = sample_rate;
+	plugin_data->window = window;
+	plugin_data->z0 = z0;
+	plugin_data->z1 = z1;
+	plugin_data->z2 = z2;
+
+}
+
+static void cleanupRetroFlange(LADSPA_Handle instance) {
+#line 84 "retro_flange_1208.xml"
+	RetroFlange *plugin_data = (RetroFlange *)instance;
+	free(plugin_data->delay_line);
+	free(plugin_data->buffer);
+	free(plugin_data->window);
+	free(plugin_data->meta_window);
+	free(plugin_data->law_tbl);
 	free(instance);
 }
 
-void connectPortRetroFlange(
+static void connectPortRetroFlange(
  LADSPA_Handle instance,
  unsigned long port,
  LADSPA_Data *data) {
@@ -86,8 +173,8 @@ void connectPortRetroFlange(
 
 	plugin = (RetroFlange *)instance;
 	switch (port) {
-	case RETROFLANGE_DELAY_DEPTH:
-		plugin->delay_depth = data;
+	case RETROFLANGE_DELAY_DEPTH_AVG:
+		plugin->delay_depth_avg = data;
 		break;
 	case RETROFLANGE_LAW_FREQ:
 		plugin->law_freq = data;
@@ -101,7 +188,7 @@ void connectPortRetroFlange(
 	}
 }
 
-LADSPA_Handle instantiateRetroFlange(
+static LADSPA_Handle instantiateRetroFlange(
  const LADSPA_Descriptor *descriptor,
  unsigned long s_rate) {
 	RetroFlange *plugin_data = (RetroFlange *)malloc(sizeof(RetroFlange));
@@ -125,6 +212,7 @@ LADSPA_Handle instantiateRetroFlange(
 	LADSPA_Data z1;
 	LADSPA_Data z2;
 
+#line 45 "retro_flange_1208.xml"
 	int i;
 	
 	sample_rate = s_rate;
@@ -185,20 +273,20 @@ LADSPA_Handle instantiateRetroFlange(
 #define RUN_ADDING    0
 #define RUN_REPLACING 1
 
-void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
+static void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	RetroFlange *plugin_data = (RetroFlange *)instance;
 
 	/* Average stall (ms) (float value) */
-	LADSPA_Data delay_depth = *(plugin_data->delay_depth);
+	const LADSPA_Data delay_depth_avg = *(plugin_data->delay_depth_avg);
 
 	/* Flange frequency (Hz) (float value) */
-	LADSPA_Data law_freq = *(plugin_data->law_freq);
+	const LADSPA_Data law_freq = *(plugin_data->law_freq);
 
 	/* Input (array of floats of length sample_count) */
-	LADSPA_Data *input = plugin_data->input;
+	const LADSPA_Data * const input = plugin_data->input;
 
 	/* Output (array of floats of length sample_count) */
-	LADSPA_Data *output = plugin_data->output;
+	LADSPA_Data * const output = plugin_data->output;
 	LADSPA_Data * buffer = plugin_data->buffer;
 	long buffer_size = plugin_data->buffer_size;
 	long count = plugin_data->count;
@@ -219,25 +307,26 @@ void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	LADSPA_Data z1 = plugin_data->z1;
 	LADSPA_Data z2 = plugin_data->z2;
 
+#line 92 "retro_flange_1208.xml"
 	long int pos;
-	int law_p = f_trunc(LIMIT((1.0f/law_freq) * sample_rate, 1, max_law_p));
+	int law_p = f_trunc(LIMIT(sample_rate / f_clamp(law_freq, 0.0001f, 100.0f), 1, max_law_p));
 	float increment;
 	float lin_int, lin_inc;
 	int track;
 	int fph;
 	LADSPA_Data out = 0.0f;
 	int i;
-	int dl_used = (delay_depth * sample_rate) / 1000;
+	const float dda_c = f_clamp(delay_depth_avg, 0.0f, 10.0f);
+	int dl_used = (dda_c * sample_rate) / 1000;
 	float inc_base = 1000.0f * (float)BASE_BUFFER;
-	
-	delay_depth *= 2.0f;
+	const float delay_depth = 2.0f * dda_c;
 	
 	for (pos = 0; pos < sample_count; pos++) {
 	        // Write into the delay line
 	        delay_line[delay_pos] = input[pos];
 	        z0 = delay_line[MOD(delay_pos - dl_used, delay_line_length)] + 0.12919609397f*z1 - 0.31050847f*z2;
 	        out = sat(z0*0.20466966f + z1*0.40933933f + z2*0.40933933f,
-	                        -0.23f, 6.3f);
+	                        -0.23f, 3.3f);
 	        z2 = z1; z1 = z0;
 	        delay_pos = (delay_pos + 1) % delay_line_length;
 	
@@ -263,8 +352,8 @@ void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	        fph = f_trunc(phase);
 	        last_phase = fph;
 	        lin_int = phase - (float)fph;
-	        out -= sat(LIN_INTERP(lin_int, buffer[(fph+1) % buffer_size],
-	         buffer[(fph+2) % buffer_size]), -0.3f, 5.7f);
+	        out += LIN_INTERP(lin_int, buffer[(fph+1) % buffer_size],
+	         buffer[(fph+2) % buffer_size]);
 	        phase += increment;
 	        lin_inc = 1.0f / (floor(phase) - last_phase + 1);
 	        lin_inc = lin_inc > 1.0f ? 1.0f : lin_inc;
@@ -275,7 +364,7 @@ void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	                 LIN_INTERP(lin_int, last_in, input[pos]);
 	        }
 	        last_in = input[pos];
-	        buffer_write(output[pos], out);
+	        buffer_write(output[pos], out * 0.707f);
 	        if (phase >= buffer_size) {
 	                phase -= buffer_size;
 	        }
@@ -307,25 +396,25 @@ void runRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 #define RUN_ADDING    1
 #define RUN_REPLACING 0
 
-void setRunAddingGainRetroFlange(LADSPA_Handle instance, LADSPA_Data gain) {
+static void setRunAddingGainRetroFlange(LADSPA_Handle instance, LADSPA_Data gain) {
 	((RetroFlange *)instance)->run_adding_gain = gain;
 }
 
-void runAddingRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
+static void runAddingRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	RetroFlange *plugin_data = (RetroFlange *)instance;
 	LADSPA_Data run_adding_gain = plugin_data->run_adding_gain;
 
 	/* Average stall (ms) (float value) */
-	LADSPA_Data delay_depth = *(plugin_data->delay_depth);
+	const LADSPA_Data delay_depth_avg = *(plugin_data->delay_depth_avg);
 
 	/* Flange frequency (Hz) (float value) */
-	LADSPA_Data law_freq = *(plugin_data->law_freq);
+	const LADSPA_Data law_freq = *(plugin_data->law_freq);
 
 	/* Input (array of floats of length sample_count) */
-	LADSPA_Data *input = plugin_data->input;
+	const LADSPA_Data * const input = plugin_data->input;
 
 	/* Output (array of floats of length sample_count) */
-	LADSPA_Data *output = plugin_data->output;
+	LADSPA_Data * const output = plugin_data->output;
 	LADSPA_Data * buffer = plugin_data->buffer;
 	long buffer_size = plugin_data->buffer_size;
 	long count = plugin_data->count;
@@ -346,25 +435,26 @@ void runAddingRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	LADSPA_Data z1 = plugin_data->z1;
 	LADSPA_Data z2 = plugin_data->z2;
 
+#line 92 "retro_flange_1208.xml"
 	long int pos;
-	int law_p = f_trunc(LIMIT((1.0f/law_freq) * sample_rate, 1, max_law_p));
+	int law_p = f_trunc(LIMIT(sample_rate / f_clamp(law_freq, 0.0001f, 100.0f), 1, max_law_p));
 	float increment;
 	float lin_int, lin_inc;
 	int track;
 	int fph;
 	LADSPA_Data out = 0.0f;
 	int i;
-	int dl_used = (delay_depth * sample_rate) / 1000;
+	const float dda_c = f_clamp(delay_depth_avg, 0.0f, 10.0f);
+	int dl_used = (dda_c * sample_rate) / 1000;
 	float inc_base = 1000.0f * (float)BASE_BUFFER;
-	
-	delay_depth *= 2.0f;
+	const float delay_depth = 2.0f * dda_c;
 	
 	for (pos = 0; pos < sample_count; pos++) {
 	        // Write into the delay line
 	        delay_line[delay_pos] = input[pos];
 	        z0 = delay_line[MOD(delay_pos - dl_used, delay_line_length)] + 0.12919609397f*z1 - 0.31050847f*z2;
 	        out = sat(z0*0.20466966f + z1*0.40933933f + z2*0.40933933f,
-	                        -0.23f, 6.3f);
+	                        -0.23f, 3.3f);
 	        z2 = z1; z1 = z0;
 	        delay_pos = (delay_pos + 1) % delay_line_length;
 	
@@ -390,8 +480,8 @@ void runAddingRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	        fph = f_trunc(phase);
 	        last_phase = fph;
 	        lin_int = phase - (float)fph;
-	        out -= sat(LIN_INTERP(lin_int, buffer[(fph+1) % buffer_size],
-	         buffer[(fph+2) % buffer_size]), -0.3f, 5.7f);
+	        out += LIN_INTERP(lin_int, buffer[(fph+1) % buffer_size],
+	         buffer[(fph+2) % buffer_size]);
 	        phase += increment;
 	        lin_inc = 1.0f / (floor(phase) - last_phase + 1);
 	        lin_inc = lin_inc > 1.0f ? 1.0f : lin_inc;
@@ -402,7 +492,7 @@ void runAddingRetroFlange(LADSPA_Handle instance, unsigned long sample_count) {
 	                 LIN_INTERP(lin_int, last_in, input[pos]);
 	        }
 	        last_in = input[pos];
-	        buffer_write(output[pos], out);
+	        buffer_write(output[pos], out * 0.707f);
 	        if (phase >= buffer_size) {
 	                phase -= buffer_size;
 	        }
@@ -432,20 +522,29 @@ void _init() {
 	LADSPA_PortDescriptor *port_descriptors;
 	LADSPA_PortRangeHint *port_range_hints;
 
+#ifdef ENABLE_NLS
+#define D_(s) dgettext(PACKAGE, s)
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
+#else
+#define D_(s) (s)
+#endif
+
+
 	retroFlangeDescriptor =
 	 (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));
 
 	if (retroFlangeDescriptor) {
 		retroFlangeDescriptor->UniqueID = 1208;
-		retroFlangeDescriptor->Label = strdup("retroFlange");
+		retroFlangeDescriptor->Label = "retroFlange";
 		retroFlangeDescriptor->Properties =
 		 0;
 		retroFlangeDescriptor->Name =
-		 strdup("Retro Flanger");
+		 D_("Retro Flanger");
 		retroFlangeDescriptor->Maker =
-		 strdup("Steve Harris <steve@plugin.org.uk>");
+		 "Steve Harris <steve@plugin.org.uk>";
 		retroFlangeDescriptor->Copyright =
-		 strdup("GPL");
+		 "GPL";
 		retroFlangeDescriptor->PortCount = 4;
 
 		port_descriptors = (LADSPA_PortDescriptor *)calloc(4,
@@ -463,22 +562,22 @@ void _init() {
 		 (const char **)port_names;
 
 		/* Parameters for Average stall (ms) */
-		port_descriptors[RETROFLANGE_DELAY_DEPTH] =
+		port_descriptors[RETROFLANGE_DELAY_DEPTH_AVG] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
-		port_names[RETROFLANGE_DELAY_DEPTH] =
-		 strdup("Average stall (ms)");
-		port_range_hints[RETROFLANGE_DELAY_DEPTH].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
-		port_range_hints[RETROFLANGE_DELAY_DEPTH].LowerBound = 0;
-		port_range_hints[RETROFLANGE_DELAY_DEPTH].UpperBound = 10;
+		port_names[RETROFLANGE_DELAY_DEPTH_AVG] =
+		 D_("Average stall (ms)");
+		port_range_hints[RETROFLANGE_DELAY_DEPTH_AVG].HintDescriptor =
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_LOW;
+		port_range_hints[RETROFLANGE_DELAY_DEPTH_AVG].LowerBound = 0;
+		port_range_hints[RETROFLANGE_DELAY_DEPTH_AVG].UpperBound = 10;
 
 		/* Parameters for Flange frequency (Hz) */
 		port_descriptors[RETROFLANGE_LAW_FREQ] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[RETROFLANGE_LAW_FREQ] =
-		 strdup("Flange frequency (Hz)");
+		 D_("Flange frequency (Hz)");
 		port_range_hints[RETROFLANGE_LAW_FREQ].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_1;
 		port_range_hints[RETROFLANGE_LAW_FREQ].LowerBound = 0.5;
 		port_range_hints[RETROFLANGE_LAW_FREQ].UpperBound = 8;
 
@@ -486,17 +585,17 @@ void _init() {
 		port_descriptors[RETROFLANGE_INPUT] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
 		port_names[RETROFLANGE_INPUT] =
-		 strdup("Input");
+		 D_("Input");
 		port_range_hints[RETROFLANGE_INPUT].HintDescriptor = 0;
 
 		/* Parameters for Output */
 		port_descriptors[RETROFLANGE_OUTPUT] =
 		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
 		port_names[RETROFLANGE_OUTPUT] =
-		 strdup("Output");
+		 D_("Output");
 		port_range_hints[RETROFLANGE_OUTPUT].HintDescriptor = 0;
 
-		retroFlangeDescriptor->activate = NULL;
+		retroFlangeDescriptor->activate = activateRetroFlange;
 		retroFlangeDescriptor->cleanup = cleanupRetroFlange;
 		retroFlangeDescriptor->connect_port = connectPortRetroFlange;
 		retroFlangeDescriptor->deactivate = NULL;
@@ -508,16 +607,8 @@ void _init() {
 }
 
 void _fini() {
-	int i;
-
 	if (retroFlangeDescriptor) {
-		free((char *)retroFlangeDescriptor->Label);
-		free((char *)retroFlangeDescriptor->Name);
-		free((char *)retroFlangeDescriptor->Maker);
-		free((char *)retroFlangeDescriptor->Copyright);
 		free((LADSPA_PortDescriptor *)retroFlangeDescriptor->PortDescriptors);
-		for (i = 0; i < retroFlangeDescriptor->PortCount; i++)
-			free((char *)(retroFlangeDescriptor->PortNames[i]));
 		free((char **)retroFlangeDescriptor->PortNames);
 		free((LADSPA_PortRangeHint *)retroFlangeDescriptor->PortRangeHints);
 		free(retroFlangeDescriptor);
